/* Copyright (c) 2009   Guy WEILER         weigu[at]weigu[dot]lu

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   Licence: GPL-3        */

#include <iostream>
#include <cstdio>
#include <lusb0_usb.h>
#include "usb_drv.h"
using namespace std;

unsigned int timeout = 2000;
unsigned int status;

struct usb_device *usb_find_device(void) {
  struct usb_bus *bus;
  struct usb_device *dev;
  usb_init();
  usb_find_busses();
  usb_find_devices();
  for (bus = usb_busses; bus; bus = bus->next) {
    for (dev = bus->devices; dev; dev = dev->next) {
      if ((dev->descriptor.idVendor == VID) && (dev->descriptor.idProduct == PID))
        return dev;
      else return 0;
    }
  }
  return 0;
}

struct usb_dev_handle *usb_claim_device(void)
{
  struct usb_device *dev;
  struct usb_dev_handle *handle;
  int conf_error, claim_error;
  dev = usb_find_device();
  if (dev == NULL) {
      cout << "Geraet nicht gefunden!";
      return NULL;
  }
  else {
    handle = usb_open(dev);
    if (handle == NULL) cout << "Geraete Handle nicht gefunden!";
    conf_error = usb_set_configuration(handle, 1);
    if ( conf_error < 0) {
      cout << "Kann Konfiguration nicht setzen (Fehlercode = " << conf_error << "!)";
      usb_close(handle);
    }
    claim_error = usb_claim_interface(handle, 0);
    if ( claim_error < 0) {
      cout << "Kann Inerface nicht anfordern (errorcode = " << claim_error << "!)";
      usb_close(handle);
    }
    return handle;
  }
}

void send_to_port(unsigned char Port) {
  struct usb_dev_handle *handle;
  handle = usb_claim_device();
  if (handle != NULL) {
    if (usb_bulk_write(handle, 0x02, (char*) &Port, 1, timeout) < 1)
      cout << "Fehler beim Schreiben aufgetreten!";  
    usb_release_interface(handle, 0);
    usb_close(handle);
  }
}


unsigned int get_once_from_adc() {
  struct usb_dev_handle *handle;
  unsigned char buf[2];
  unsigned int adcvalue;
  handle = usb_claim_device();
  if (handle != NULL) {
    if (usb_bulk_read(handle, 0x81, (char*) buf, 2, timeout) < (2))
      cout << "Es ist ein Fehler beim Lesen aufgetreten!";
    adcvalue = (unsigned int) (buf[1]*256)+buf[0];

    usb_release_interface(handle, 0);
    usb_close(handle);
    return adcvalue;
  }

}

